NAME = "clipped"
VERSION = "0.1.0"
DESC = "Common shortcuts and utils."
URL = "https://github.com/mmourafiq/clipped"
AUTHOR = "Mourad Mourafiq"
EMAIL = "mourad.mourafiq@gmail.com"
LICENSE = "Apache 2.0"
